@@ -1,20 +1,13 @@
-use darling::{
-    ast::Data,
-    util::{Ignored, SpannedValue},
-    FromDeriveInput, FromField,
-};
+use darling::{ast::Data, util::Ignored, FromDeriveInput, FromField};
 use proc_macro2::{Ident, TokenStream};
 use quote::quote;
 use syn::{ext::IdentExt, Attribute, DeriveInput, Error, GenericParam, Generics, Type};
 
 use crate::{
-    common_args::{
-        ConcreteType, DefaultValue, MaximumValidator, MinimumValidator, RenameRule, RenameRuleExt,
-        RenameTarget,
-    },
+    common_args::{ConcreteType, DefaultValue, RenameRule, RenameRuleExt, RenameTarget},
     error::GeneratorResult,
     utils::{get_crate_name, get_summary_and_description, optional_literal},
-    validators::HasValidators,
+    validators::Validators,
 };
 
 #[derive(FromField)]
@@ -35,29 +28,10 @@ struct ObjectField {
     write_only: bool,
     #[darling(default)]
     read_only: bool,
-
     #[darling(default)]
-    multiple_of: Option<SpannedValue<f64>>,
-    #[darling(default)]
-    maximum: Option<SpannedValue<MaximumValidator>>,
-    #[darling(default)]
-    minimum: Option<SpannedValue<MinimumValidator>>,
-    #[darling(default)]
-    max_length: Option<SpannedValue<usize>>,
-    #[darling(default)]
-    min_length: Option<SpannedValue<usize>>,
-    #[darling(default)]
-    pattern: Option<SpannedValue<String>>,
-    #[darling(default)]
-    max_items: Option<SpannedValue<usize>>,
-    #[darling(default)]
-    min_items: Option<SpannedValue<usize>>,
-    #[darling(default)]
-    unique_items: bool,
+    validator: Option<Validators>,
 }
 
-impl_has_validators!(ObjectField);
-
 #[derive(FromDeriveInput)]
 #[darling(attributes(oai), forward_attrs(doc))]
 struct ObjectArgs {
@@ -146,10 +120,9 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
         let (field_title, field_description) = get_summary_and_description(&field.attrs)?;
         let field_title = optional_literal(&field_title);
         let field_description = optional_literal(&field_description);
-        let validators_checker = field
-            .validators()
-            .create_obj_field_checker(&crate_name, &field_name)?;
-        let validators_update_meta = field.validators().create_update_meta(&crate_name)?;
+        let validators = field.validator.clone().unwrap_or_default();
+        let validators_checker = validators.create_obj_field_checker(&crate_name, &field_name)?;
+        let validators_update_meta = validators.create_update_meta(&crate_name)?;
 
         fields.push(field_ident);
 
@@ -177,7 +150,7 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                     #[allow(non_snake_case)]
                     let #field_ident: #field_ty = {
                         match obj.get(#field_name).cloned().unwrap_or_default() {
-                            #crate_name::serde_json::Value::Null => #default_value,
+                            #crate_name::__private::serde_json::Value::Null => #default_value,
                             value => {
                                 let value = #crate_name::types::ParseFromJSON::parse_from_json(value).map_err(#crate_name::types::ParseError::propagate)?;
                                 #validators_checker
@@ -223,6 +196,7 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
         register_types.push(quote!(<#field_ty>::register(registry);));
 
         meta_fields.push(quote! {{
+            let original_schema = <#field_ty as #crate_name::types::Type>::schema_ref();
             let patch_schema = {
                 let mut schema = #crate_name::registry::MetaSchema::ANY;
                 schema.default = #field_meta_default;
@@ -240,7 +214,7 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                 schema
             };
 
-            (#field_name, <#field_ty as #crate_name::types::Type>::schema_ref().merge(patch_schema))
+            (#field_name, original_schema.merge(patch_schema))
         }});
 
         required_fields.push(quote! {
@@ -288,20 +262,22 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                 quote!(#crate_name::registry::MetaSchemaRef::Reference(#oai_typename)),
                 quote! {
                     #(#register_types)*
-                    registry.create_schema(#oai_typename, |registry| #meta)
+                    registry.create_schema::<Self, _>(#oai_typename, |registry| #meta)
                 },
             )
         };
 
         quote! {
             impl #impl_generics #crate_name::types::Type for #ident #ty_generics #where_clause {
-                const NAME: #crate_name::types::TypeName = #crate_name::types::TypeName::Normal {
-                    ty: #oai_typename,
-                    format: ::std::option::Option::None,
-                };
                 const IS_REQUIRED: bool = true;
 
-                type ValueType = Self;
+                type RawValueType = Self;
+
+                type RawElementValueType = Self;
+
+                fn name() -> ::std::borrow::Cow<'static, str> {
+                    ::std::convert::Into::into(#oai_typename)
+                }
 
                 fn schema_ref() -> #crate_name::registry::MetaSchemaRef {
                     #fn_schema_ref
@@ -311,15 +287,19 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                     #fn_register
                 }
 
-                fn as_value(&self) -> ::std::option::Option<&Self> {
+                fn as_raw_value(&self) -> ::std::option::Option<&Self::RawValueType> {
                     ::std::option::Option::Some(self)
                 }
+
+                fn raw_element_iter<'a>(&'a self) -> ::std::boxed::Box<dyn ::std::iter::Iterator<Item = &'a Self::RawElementValueType> + 'a> {
+                    ::std::boxed::Box::new(::std::iter::IntoIterator::into_iter(self.as_raw_value()))
+                }
             }
 
             impl #impl_generics #crate_name::types::ParseFromJSON for #ident #ty_generics #where_clause {
-                fn parse_from_json(value: #crate_name::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> {
+                fn parse_from_json(value: #crate_name::__private::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> {
                     match value {
-                        #crate_name::serde_json::Value::Object(obj) => {
+                        #crate_name::__private::serde_json::Value::Object(obj) => {
                             #(#deserialize_fields)*
                             ::std::result::Result::Ok(Self { #(#fields),* })
                         }
@@ -329,23 +309,23 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
             }
 
             impl #impl_generics #crate_name::types::ToJSON for #ident #ty_generics #where_clause {
-                fn to_json(&self) -> #crate_name::serde_json::Value {
-                    let mut object = ::#crate_name::serde_json::Map::new();
+                fn to_json(&self) -> #crate_name::__private::serde_json::Value {
+                    let mut object = ::#crate_name::__private::serde_json::Map::new();
                     #(#serialize_fields)*
-                    #crate_name::serde_json::Value::Object(object)
+                    #crate_name::__private::serde_json::Value::Object(object)
                 }
             }
 
-            impl #impl_generics #crate_name::serde::Serialize for #ident #ty_generics #where_clause {
-                fn serialize<S: #crate_name::serde::Serializer>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error> {
+            impl #impl_generics #crate_name::__private::serde::Serialize for #ident #ty_generics #where_clause {
+                fn serialize<S: #crate_name::__private::serde::Serializer>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error> {
                     #crate_name::types::ToJSON::to_json(self).serialize(serializer)
                 }
             }
 
-            impl #de_impl_generics #crate_name::serde::Deserialize<'de> for #ident #ty_generics #where_clause {
-                fn deserialize<D: #crate_name::serde::Deserializer<'de>>(deserializer: D) -> ::std::result::Result<Self, D::Error> {
-                    let value: #crate_name::serde_json::Value = #crate_name::serde::de::Deserialize::deserialize(deserializer)?;
-                    #crate_name::types::ParseFromJSON::parse_from_json(value).map_err(|err| #crate_name::serde::de::Error::custom(err.into_message()))
+            impl #de_impl_generics #crate_name::__private::serde::Deserialize<'de> for #ident #ty_generics #where_clause {
+                fn deserialize<D: #crate_name::__private::serde::Deserializer<'de>>(deserializer: D) -> ::std::result::Result<Self, D::Error> {
+                    let value: #crate_name::__private::serde_json::Value = #crate_name::__private::serde::de::Deserialize::deserialize(deserializer)?;
+                    #crate_name::types::ParseFromJSON::parse_from_json(value).map_err(|err| #crate_name::__private::serde::de::Error::custom(err.into_message()))
                 }
             }
         }
@@ -354,14 +334,14 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
 
         code.push(quote! {
             impl #impl_generics #ident #ty_generics #where_clause {
-                fn __internal_register(registry: &mut #crate_name::registry::Registry) where Self: #crate_name::types::Type {
+                fn __internal_register(name: &'static str, registry: &mut #crate_name::registry::Registry) where Self: #crate_name::types::Type {
                     #(#register_types)*
-                    registry.create_schema(Self::NAME.type_name(), |registry| #meta);
+                    registry.create_schema::<Self, _>(name, |registry| #meta);
                 }
 
-                fn __internal_parse_from_json(value: #crate_name::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> where Self: #crate_name::types::Type {
+                fn __internal_parse_from_json(value: #crate_name::__private::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> where Self: #crate_name::types::Type {
                     match value {
-                        #crate_name::serde_json::Value::Object(obj) => {
+                        #crate_name::__private::serde_json::Value::Object(obj) => {
                             #(#deserialize_fields)*
                             ::std::result::Result::Ok(Self { #(#fields),* })
                         }
@@ -369,10 +349,10 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                     }
                 }
 
-                fn __internal_to_json(&self) -> #crate_name::serde_json::Value where Self: #crate_name::types::Type {
+                fn __internal_to_json(&self) -> #crate_name::__private::serde_json::Value where Self: #crate_name::types::Type {
                     let mut object = ::serde_json::Map::new();
                     #(#serialize_fields)*
-                    #crate_name::serde_json::Value::Object(object)
+                    #crate_name::__private::serde_json::Value::Object(object)
                 }
             }
         });
@@ -384,16 +364,18 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
 
             let expanded = quote! {
                 impl #crate_name::types::Type for #concrete_type {
-                    const NAME: #crate_name::types::TypeName = #crate_name::types::TypeName::Normal {
-                        ty: #oai_typename,
-                        format: ::std::option::Option::None,
-                    };
                     const IS_REQUIRED: bool = true;
 
-                    type ValueType = Self;
+                    type RawValueType = Self;
 
-                    fn as_value(&self) -> Option<&Self> {
-                        Some(self)
+                    type RawElementValueType = Self;
+
+                    fn name() -> ::std::borrow::Cow<'static, str> {
+                        ::std::convert::Into::into(#oai_typename)
+                    }
+
+                    fn as_raw_value(&self) -> Option<&Self::RawValueType> {
+                        ::std::option::Option::Some(self)
                     }
 
                     fn schema_ref() -> #crate_name::registry::MetaSchemaRef {
@@ -401,32 +383,36 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
                     }
 
                     fn register(registry: &mut #crate_name::registry::Registry) {
-                        Self::__internal_register(registry);
+                        Self::__internal_register(#oai_typename, registry);
+                    }
+
+                    fn raw_element_iter<'a>(&'a self) -> ::std::boxed::Box<dyn ::std::iter::Iterator<Item = &'a Self::RawElementValueType> + 'a> {
+                        ::std::boxed::Box::new(::std::iter::IntoIterator::into_iter(self.as_raw_value()))
                     }
                 }
 
                 impl #crate_name::types::ParseFromJSON for #concrete_type {
-                    fn parse_from_json(value: #crate_name::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> {
+                    fn parse_from_json(value: #crate_name::__private::serde_json::Value) -> ::std::result::Result<Self, #crate_name::types::ParseError<Self>> {
                         Self::__internal_parse_from_json(value)
                     }
                 }
 
                 impl #crate_name::types::ToJSON for #concrete_type {
-                    fn to_json(&self) -> #crate_name::serde_json::Value {
+                    fn to_json(&self) -> #crate_name::__private::serde_json::Value {
                         Self::__internal_to_json(self)
                     }
                 }
 
-                impl #crate_name::serde::Serialize for #concrete_type {
-                    fn serialize<S: #crate_name::serde::Serializer>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error> {
+                impl #crate_name::__private::serde::Serialize for #concrete_type {
+                    fn serialize<S: #crate_name::__private::serde::Serializer>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error> {
                         #crate_name::types::ToJSON::to_json(self).serialize(serializer)
                     }
                 }
 
-                impl<'de> #crate_name::serde::Deserialize<'de> for #concrete_type {
-                    fn deserialize<D: #crate_name::serde::Deserializer<'de>>(deserializer: D) -> ::std::result::Result<Self, D::Error> {
-                        let value: #crate_name::serde_json::Value = #crate_name::serde::de::Deserialize::deserialize(deserializer)?;
-                        #crate_name::types::ParseFromJSON::parse_from_json(value).map_err(|err| #crate_name::serde::de::Error::custom(err.into_message()))
+                impl<'de> #crate_name::__private::serde::Deserialize<'de> for #concrete_type {
+                    fn deserialize<D: #crate_name::__private::serde::Deserializer<'de>>(deserializer: D) -> ::std::result::Result<Self, D::Error> {
+                        let value: #crate_name::__private::serde_json::Value = #crate_name::__private::serde::de::Deserialize::deserialize(deserializer)?;
+                        #crate_name::types::ParseFromJSON::parse_from_json(value).map_err(|err| #crate_name::__private::serde::de::Error::custom(err.into_message()))
                     }
                 }
             };
